@@ -8,8 +8,14 @@ from ibis.backends.base.sql.compiler import (
     SelectBuilder,
     TableSetFormatter,
 )
+from ibis.backends.base.sql.compiler.query_builder import Union
+from ibis.backends.clickhouse.registry import operation_registry
 
-from .registry import operation_registry
+
+class ClickhouseUnion(Union):
+    @staticmethod
+    def keyword(distinct):
+        return 'UNION DISTINCT' if distinct else 'UNION ALL'
 
 
 class ClickhouseSelectBuilder(SelectBuilder):
@@ -57,9 +63,14 @@ class ClickhouseTableSetFormatter(TableSetFormatter):
 
     _join_names = {
         ops.InnerJoin: 'ALL INNER JOIN',
-        ops.LeftJoin: 'ALL LEFT JOIN',
+        ops.LeftJoin: 'ALL LEFT OUTER JOIN',
+        ops.RightJoin: 'ALL RIGHT OUTER JOIN',
+        ops.OuterJoin: 'ALL FULL OUTER JOIN',
+        ops.CrossJoin: 'CROSS JOIN',
+        ops.LeftSemiJoin: 'LEFT SEMI JOIN',
+        ops.LeftAntiJoin: 'LEFT ANTI JOIN',
         ops.AnyInnerJoin: 'ANY INNER JOIN',
-        ops.AnyLeftJoin: 'ANY LEFT JOIN',
+        ops.AnyLeftJoin: 'ANY LEFT OUTER JOIN',
     }
 
     _non_equijoin_supported = False
@@ -111,3 +122,4 @@ class ClickhouseCompiler(Compiler):
     table_set_formatter_class = ClickhouseTableSetFormatter
     select_builder_class = ClickhouseSelectBuilder
     select_class = ClickhouseSelect
+    union_class = ClickhouseUnion
