@@ -1,5 +1,6 @@
 import pytest
 from pandas import Timestamp
+from pytest import param
 
 import ibis
 from ibis import literal as L
@@ -16,12 +17,42 @@ pytest.importorskip("clickhouse_driver")
     ],
 )
 def test_timestamp_literals(con, translate, expr):
-    expected = "toDateTime('2015-01-01 12:34:56')"
+    expected = "toDateTime('2015-01-01T12:34:56')"
 
     assert translate(expr) == expected
     assert con.execute(expr) == Timestamp('2015-01-01 12:34:56')
 
 
+@pytest.mark.parametrize(
+    ("expr", "expected"),
+    [
+        param(
+            ibis.timestamp('2015-01-01 12:34:56.789'),
+            "toDateTime64('2015-01-01T12:34:56.789000', 3)",
+            id="millis",
+        ),
+        param(
+            ibis.timestamp('2015-01-01 12:34:56.789321'),
+            "toDateTime64('2015-01-01T12:34:56.789321', 6)",
+            id="micros",
+        ),
+        param(
+            ibis.timestamp('2015-01-01 12:34:56.789 UTC'),
+            "toDateTime64('2015-01-01T12:34:56.789000', 3, 'UTC')",
+            id="millis_tz",
+        ),
+        param(
+            ibis.timestamp('2015-01-01 12:34:56.789321 UTC'),
+            "toDateTime64('2015-01-01T12:34:56.789321', 6, 'UTC')",
+            id="micros_tz",
+        ),
+    ],
+)
+def test_subsecond_timestamp_literals(con, translate, expr, expected):
+    assert translate(expr) == expected
+    assert con.execute(expr) == expr.op().value
+
+
 @pytest.mark.parametrize(
     ('value', 'expected'),
     [
