@@ -193,7 +193,7 @@ def test_complex_array_expr_projection(db, alltypes):
 
     query = ibis.clickhouse.compile(expr2)
     name = expr2.get_name()
-    expected = f"""SELECT CAST(`string_col` AS Float64) AS `{name}`
+    expected = f"""SELECT CAST(`string_col` AS Nullable(Float64)) AS `{name}`
 FROM (
   SELECT `string_col`, count(*) AS `count`
   FROM {db.name}.`functional_alltypes`
