@@ -2,6 +2,7 @@ from typing import Any, Callable, Dict, List, Optional, Tuple, Type, Union
 
 import dask.dataframe as dd
 import dask.delayed
+import numpy as np
 import pandas as pd
 from dask.dataframe.groupby import SeriesGroupBy
 
@@ -37,10 +38,26 @@ def register_types_to_dispatcher(
             dispatcher.register(ibis_op, *types_to_register)(fn)
 
 
-def make_meta_series(dtype, name=None, index_name=None):
+def make_meta_series(
+    dtype: np.dtype,
+    name: Optional[str] = None,
+    meta_index: Optional[pd.Index] = None,
+):
+    if isinstance(meta_index, pd.MultiIndex):
+        index_names = meta_index.names
+        series_index = pd.MultiIndex(
+            levels=[[]] * len(index_names),
+            codes=[[]] * len(index_names),
+            names=index_names,
+        )
+    elif isinstance(meta_index, pd.Index):
+        series_index = pd.Index([], name=meta_index.name)
+    else:
+        series_index = pd.Index([])
+
     return pd.Series(
         [],
-        index=pd.Index([], name=index_name),
+        index=series_index,
         dtype=dtype,
         name=name,
     )
@@ -123,7 +140,7 @@ def coerce_to_output(
     elif isinstance(result, (pd.Series, dd.Series)):
         # Series from https://github.com/ibis-project/ibis/issues/2711
         return result.rename(result_name)
-    elif isinstance(expr, ir.ScalarExpr):
+    elif isinstance(expr, ir.Scalar):
         if isinstance(result, dd.core.Scalar):
             # wrap the scalar in a series
             out_dtype = _pandas_dtype_from_dd_scalar(result)
@@ -209,15 +226,17 @@ def _coerce_to_dataframe(
         # NOTE - We add a detailed meta here so we do not drop the key index
         # downstream. This seems to be fixed in versions of dask > 2020.12.0
         dtypes = map(ibis_dtype_to_pandas, types)
-
         series = [
             data.apply(
                 _select_item_in_iter,
                 selection=i,
-                meta=make_meta_series(dtype, index_name=data.index.name),
+                meta=make_meta_series(
+                    dtype, meta_index=data._meta_nonempty.index
+                ),
             )
             for i, dtype in enumerate(dtypes)
         ]
+
         result = dd.concat(series, axis=1)
 
     elif isinstance(data, (tuple, list)):
@@ -254,14 +273,14 @@ def safe_concat(dfs: List[Union[dd.Series, dd.DataFrame]]) -> dd.DataFrame:
     operate on objects with equal lengths, otherwise it will raise a
     ValueError in `concat_and_check`.
 
-    See https://github.com/dask/dask/blob/2c2e837674895cafdb0612be81250ef2657d947e/dask/dataframe/multi.py#L907 # noqa
+    See https://github.com/dask/dask/blob/2c2e837674895cafdb0612be81250ef2657d947e/dask/dataframe/multi.py#L907.
 
     Note - Repeatedly joining dataframes is likely to be quite slow, but this
     should be hit rarely in real usage. A situtation that triggeres this slow
     path is aggregations where aggregations return different numbers of rows
     (see `test_aggregation_group_by` for a specific example).
     TODO - performance.
-    """
+    """  # noqa: E501
     if len(dfs) == 1:
         maybe_df = dfs[0]
         if isinstance(maybe_df, dd.Series):
@@ -447,7 +466,7 @@ def is_row_order_preserving(exprs) -> bool:
     """
 
     def _is_row_order_preserving(expr: ir.Expr):
-        if isinstance(expr.op(), (ops.Reduction, ops.WindowOp)):
+        if isinstance(expr.op(), (ops.Reduction, ops.Window)):
             return (lin.halt, False)
         else:
             return (lin.proceed, True)
