@@ -1,6 +1,10 @@
 import pytest
 
-from ...execution.util import assert_identical_grouping_keys
+pytest.importorskip("dask.dataframe")
+
+from ibis.backends.dask.execution.util import (  # noqa: E402
+    assert_identical_grouping_keys,
+)
 
 
 @pytest.mark.parametrize(
