@@ -1,3 +1,4 @@
+import os
 from posixpath import join as pjoin
 
 import pandas as pd
@@ -6,9 +7,12 @@ import pytest
 
 import ibis
 import ibis.util as util
-from ibis.backends.impala.compat import ImpylaError
 from ibis.tests.util import assert_equal
 
+pytest.importorskip("impala")
+
+from ibis.backends.impala.compat import ImpylaError  # noqa: E402
+
 
 @pytest.fixture
 def df():
@@ -146,16 +150,6 @@ def test_create_partitioned_table_from_expr(con, alltypes):
         con.drop_table(name, force=True)
 
 
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_insert_overwrite_partition():
-    assert False
-
-
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_dynamic_partitioning():
-    assert False
-
-
 def test_add_drop_partition_no_location(con, temp_table):
     schema = ibis.schema(
         [('foo', 'string'), ('year', 'int32'), ('month', 'int16')]
@@ -175,6 +169,7 @@ def test_add_drop_partition_no_location(con, temp_table):
     assert len(table.partitions()) == 1
 
 
+@pytest.mark.hdfs
 def test_add_drop_partition_owned_by_impala(hdfs, con, temp_table):
     schema = ibis.schema(
         [('foo', 'string'), ('year', 'int32'), ('month', 'int16')]
@@ -190,8 +185,9 @@ def test_add_drop_partition_owned_by_impala(hdfs, con, temp_table):
     basename = util.guid()
     path = f'/tmp/{subdir}/{basename}'
 
-    hdfs.mkdir(f'/tmp/{subdir}')
-    hdfs.chown(f'/tmp/{subdir}', owner='impala', group='supergroup')
+    parent = os.path.dirname(path)
+    hdfs.mkdir(parent, create_parents=True)
+    hdfs.chown(parent, owner='impala', group='supergroup')
 
     table.add_partition(part, location=path)
 
@@ -224,11 +220,7 @@ def test_add_drop_partition_hive_bug(con, temp_table):
     assert len(table.partitions()) == 1
 
 
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_set_partition_location():
-    assert False
-
-
+@pytest.mark.hdfs
 def test_load_data_partition(con, hdfs, tmp_dir, unpart_t, df, temp_table):
     part_keys = ['year', 'month']
 
@@ -262,7 +254,7 @@ def test_load_data_partition(con, hdfs, tmp_dir, unpart_t, df, temp_table):
         part_t.alter_partition(part, format='text', serde_properties=csv_props)
         part_t.load_data(chunk_path, partition=part)
 
-    hdfs.rmdir(hdfs_dir)
+    hdfs.rm(hdfs_dir, recursive=True)
     verify_partitioned_table(part_t, df, unique_keys)
 
 
@@ -279,13 +271,3 @@ def verify_partitioned_table(part_t, df, unique_keys):
 
     # allow for the total line
     assert len(parts) == len(unique_keys) + 1
-
-
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_drop_partition():
-    assert False
-
-
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_repartition_automated():
-    assert False
