@@ -3,10 +3,11 @@ import pytest
 import ibis
 import ibis.common.exceptions as com
 from ibis import window
+from ibis.backends.impala.compiler import ImpalaCompiler
 from ibis.expr.window import rows_with_max_lookback
 from ibis.tests.util import assert_equal
 
-from ..compiler import ImpalaCompiler
+pytest.importorskip("impala")
 
 
 @pytest.fixture
@@ -268,14 +269,3 @@ SELECT lag(`f` - lag(`f`) OVER (PARTITION BY `g` ORDER BY `f`)) \
 OVER (PARTITION BY `g` ORDER BY `f`) AS `foo`
 FROM ibis_testing.`alltypes`"""
     assert_sql_equal(expr, expected)
-
-
-@pytest.mark.xfail
-def test_math_on_windowed_expr():
-    # Window clause may not be found at top level of expression
-    assert False
-
-
-@pytest.mark.xfail
-def test_group_by_then_different_sort_orders():
-    assert False
