@@ -1,3 +1,4 @@
+import sqlalchemy as sa
 import sqlalchemy.dialects.mysql as mysql
 import toolz
 
@@ -28,6 +29,8 @@ class MySQLExprTranslator(AlchemyExprTranslator):
             dt.String: mysql.VARCHAR,
         }
     )
+    _bool_aggs_need_cast_to_int32 = False
+    integer_to_timestamp = sa.func.from_unixtime
 
 
 rewrites = MySQLExprTranslator.rewrites
