@@ -1,13 +1,12 @@
+import functools
 import operator
 
 import pandas as pd
 
 import ibis.expr.operations as ops
-import ibis.util
-
-from ..core import execute
-from ..dispatch import execute_node
-from ..execution import constants
+from ibis.backends.pandas.core import execute
+from ibis.backends.pandas.dispatch import execute_node
+from ibis.backends.pandas.execution import constants
 
 
 def _compute_join_column(column_expr, **kwargs):
@@ -21,8 +20,8 @@ def _compute_join_column(column_expr, **kwargs):
     return new_column, root_table
 
 
-@execute_node.register(ops.CrossJoin, pd.DataFrame, pd.DataFrame)
-def execute_cross_join(op, left, right, **kwargs):
+@execute_node.register(ops.CrossJoin, pd.DataFrame, pd.DataFrame, tuple)
+def execute_cross_join(op, left, right, predicates, **kwargs):
     """Execute a cross join in pandas.
 
     Notes
@@ -32,43 +31,67 @@ def execute_cross_join(op, left, right, **kwargs):
     by cross join.
 
     """
-    # generate a unique name for the temporary join key
-    key = f"cross_join_{ibis.util.guid()}"
-    join_key = {key: True}
-    new_left = left.assign(**join_key)
-    new_right = right.assign(**join_key)
-
-    # inner/outer doesn't matter because every row matches every other row
-    result = pd.merge(
-        new_left,
-        new_right,
-        how='inner',
-        on=key,
+    assert not predicates, "cross join predicates must be empty"
+    return pd.merge(
+        left,
+        right,
+        how='cross',
         copy=False,
         suffixes=constants.JOIN_SUFFIXES,
     )
 
-    # remove the generated key
-    del result[key]
 
-    return result
+def _get_semi_anti_join_filter(op, left, right, predicates, **kwargs):
+    left_on, right_on = _construct_join_predicate_columns(
+        op,
+        predicates,
+        **kwargs,
+    )
+    inner = pd.merge(
+        left,
+        right,
+        how="inner",
+        left_on=left_on,
+        right_on=right_on,
+        suffixes=constants.JOIN_SUFFIXES,
+    )
+    predicates = [left.loc[:, key].isin(inner.loc[:, key]) for key in left_on]
+    return functools.reduce(operator.and_, predicates)
 
 
-@execute_node.register(ops.Join, pd.DataFrame, pd.DataFrame)
-def execute_join(op, left, right, **kwargs):
-    op_type = type(op)
+@execute_node.register(ops.LeftSemiJoin, pd.DataFrame, pd.DataFrame, tuple)
+def execute_left_semi_join(op, left, right, predicates, **kwargs):
+    """Execute a left semi join in pandas."""
+    inner_filt = _get_semi_anti_join_filter(
+        op,
+        left,
+        right,
+        predicates,
+        **kwargs,
+    )
+    return left.loc[inner_filt, :]
 
-    try:
-        how = constants.JOIN_TYPES[op_type]
-    except KeyError:
-        raise NotImplementedError(f'{op_type.__name__} not supported')
 
+@execute_node.register(ops.LeftAntiJoin, pd.DataFrame, pd.DataFrame, tuple)
+def execute_left_anti_join(op, left, right, predicates, **kwargs):
+    """Execute a left anti join in pandas."""
+    inner_filt = _get_semi_anti_join_filter(
+        op,
+        left,
+        right,
+        predicates,
+        **kwargs,
+    )
+    return left.loc[~inner_filt, :]
+
+
+def _construct_join_predicate_columns(op, predicates, **kwargs):
     left_op = op.left.op()
     right_op = op.right.op()
 
     on = {left_op: [], right_op: []}
 
-    for predicate in map(operator.methodcaller('op'), op.predicates):
+    for predicate in map(operator.methodcaller('op'), predicates):
         if not isinstance(predicate, ops.Equals):
             raise TypeError(
                 'Only equality join predicates supported with pandas'
@@ -82,30 +105,45 @@ def execute_join(op, left, right, **kwargs):
             predicate.right, **kwargs
         )
         on[right_pred_root].append(new_right_column)
+    return on[left_op], on[right_op]
+
+
+@execute_node.register(ops.Join, pd.DataFrame, pd.DataFrame, tuple)
+def execute_join(op, left, right, predicates, **kwargs):
+    op_type = type(op)
+
+    try:
+        how = constants.JOIN_TYPES[op_type]
+    except KeyError:
+        raise NotImplementedError(f'{op_type.__name__} not supported')
+
+    left_on, right_on = _construct_join_predicate_columns(
+        op, predicates, **kwargs
+    )
 
     df = pd.merge(
         left,
         right,
         how=how,
-        left_on=on[left_op],
-        right_on=on[right_op],
+        left_on=left_on,
+        right_on=right_on,
         suffixes=constants.JOIN_SUFFIXES,
     )
     return df
 
 
-@execute_node.register(ops.MaterializedJoin, pd.DataFrame)
-def execute_materialized_join(op, df, **kwargs):
-    return df
-
-
 @execute_node.register(
-    ops.AsOfJoin, pd.DataFrame, pd.DataFrame, (pd.Timedelta, type(None))
+    ops.AsOfJoin,
+    pd.DataFrame,
+    pd.DataFrame,
+    tuple,
+    (pd.Timedelta, type(None)),
+    tuple,
 )
-def execute_asof_join(op, left, right, tolerance, **kwargs):
+def execute_asof_join(op, left, right, by, tolerance, predicates, **kwargs):
     overlapping_columns = frozenset(left.columns) & frozenset(right.columns)
-    left_on, right_on = _extract_predicate_names(op.predicates)
-    left_by, right_by = _extract_predicate_names(op.by)
+    left_on, right_on = _extract_predicate_names(predicates)
+    left_by, right_by = _extract_predicate_names(by)
     _validate_columns(
         overlapping_columns, left_on, right_on, left_by, right_by
     )
@@ -129,8 +167,8 @@ def _extract_predicate_names(predicates):
             raise TypeError(
                 'Only equality join predicates supported with pandas'
             )
-        left_name = predicate.left._name
-        right_name = predicate.right._name
+        left_name = predicate.left.get_name()
+        right_name = predicate.right.get_name()
         lefts.append(left_name)
         rights.append(right_name)
     return lefts, rights
