@@ -7,16 +7,15 @@ from pandas.core.groupby import SeriesGroupBy
 import ibis.expr.datatypes as dt
 import ibis.expr.operations as ops
 from ibis.backends.base import BaseBackend
-from ibis.expr.scope import Scope
-
-from ..core import (
+from ibis.backends.pandas.core import (
     date_types,
     integer_types,
     numeric_types,
     timedelta_types,
     timestamp_types,
 )
-from ..dispatch import execute_node, pre_execute
+from ibis.backends.pandas.dispatch import execute_node, pre_execute
+from ibis.expr.scope import Scope
 
 
 @execute_node.register(ops.Strftime, pd.Timestamp, str)
@@ -55,7 +54,11 @@ def execute_extract_millisecond_series(op, data, **kwargs):
 
 @execute_node.register(ops.ExtractEpochSeconds, (pd.Timestamp, pd.Series))
 def execute_epoch_seconds(op, data, **kwargs):
-    return data.astype('int64') // int(1e9)
+    # older versions of dask do not have a view method, so use astype
+    # instead
+    convert = getattr(data, "view", data.astype)
+    series = convert(np.int64)
+    return (series // 1_000_000_000).astype(np.int32)
 
 
 @execute_node.register(
