@@ -1,8 +1,7 @@
-from typing import Optional
-
 import pandas as pd
 import pandas.testing as tm
 import pytest
+from packaging.version import parse as vparse
 
 import ibis
 import ibis.common.exceptions as com
@@ -16,7 +15,7 @@ from ibis.expr.timecontext import (
     compare_timecontext,
     construct_time_context_aware_series,
 )
-from ibis.expr.types import TimeContext
+from ibis.expr.typing import TimeContext
 
 
 class CustomAsOfJoin(ops.AsOfJoin):
@@ -69,10 +68,11 @@ def test_bad_timecontext(time_table, t):
 def test_bad_call_to_adjust_context():
     op = "not_a_node"
     context = (pd.Timestamp('20170101'), pd.Timestamp('20170103'))
+    scope = Scope()
     with pytest.raises(
         com.IbisError, match=r".*Unsupported input type for adjust context.*"
     ):
-        adjust_context(op, context)
+        adjust_context(op, scope, context)
 
 
 def test_compare_timecontext():
@@ -243,6 +243,11 @@ def test_context_adjustment_multi_window(time_table, time_df3):
     tm.assert_series_equal(result["v2"], expected_win_2)
 
 
+@pytest.mark.xfail(
+    condition=vparse("1.4") <= vparse(pd.__version__) < vparse("1.4.2"),
+    raises=ValueError,
+    reason="https://github.com/pandas-dev/pandas/pull/44068",
+)
 def test_context_adjustment_window_groupby_id(time_table, time_df3):
     """This test case is meant to test trim_window_result method
     in pandas/execution/window.py to see if it could trim Series
@@ -278,8 +283,8 @@ def test_adjust_context_scope(time_keyed_left, time_keyed_right):
     @adjust_context.register(CustomAsOfJoin)
     def adjust_context_custom_asof_join(
         op: ops.AsOfJoin,
+        scope: Scope,
         timecontext: TimeContext,
-        scope: Optional[Scope] = None,
     ) -> TimeContext:
         """Confirms that `scope` is passed in."""
         assert scope is not None
@@ -318,8 +323,8 @@ def test_adjust_context_complete_shift(
     @adjust_context.register(CustomAsOfJoin)
     def adjust_context_custom_asof_join(
         op: ops.AsOfJoin,
+        scope: Scope,
         timecontext: TimeContext,
-        scope: Optional[Scope] = None,
     ) -> TimeContext:
         """Shifts both the begin and end in the same direction."""
 
