@@ -1,10 +1,10 @@
 import numpy as np
 import pandas as pd
 import pytest
-from pandas.util import testing as tm
+from pandas import testing as tm
 from pytest import param
 
-from ..aggcontext import Summarize, window_agg_udf
+from ibis.backends.pandas.aggcontext import Summarize, window_agg_udf
 
 df = pd.DataFrame(
     {
