@@ -12,7 +12,7 @@ from ibis.backends.base.sql.alchemy.registry import _geospatial_functions
 from ibis.backends.postgres.registry import operation_registry
 
 
-class PostgresUDFNode(ops.ValueOp):
+class PostgresUDFNode(ops.Value):
     output_shape = rlz.shape_like("args")
 
 
@@ -27,6 +27,7 @@ class PostgreSQLExprTranslator(AlchemyExprTranslator):
             dt.Float64: postgresql.DOUBLE_PRECISION,
         }
     )
+    _has_reduction_filter_syntax = True
 
 
 rewrites = PostgreSQLExprTranslator.rewrites
