@@ -1,6 +1,7 @@
 import functools
 import itertools
 import locale
+import operator
 import platform
 import re
 import string
@@ -8,11 +9,7 @@ import warnings
 
 import sqlalchemy as sa
 from sqlalchemy.dialects import postgresql
-from sqlalchemy.ext.compiler import compiles
-from sqlalchemy.sql import expression
-from sqlalchemy.sql.functions import GenericFunction
 
-import ibis
 import ibis.common.exceptions as com
 import ibis.common.geospatial as geo
 import ibis.expr.datatypes as dt
@@ -23,11 +20,9 @@ import ibis.expr.types as ir
 from ibis.backends.base.sql.alchemy import (
     fixed_arity,
     get_sqla_table,
-    infix_op,
     sqlalchemy_operation_registry,
     sqlalchemy_window_functions_registry,
     unary,
-    variance_reduction,
 )
 from ibis.backends.base.sql.alchemy.datatypes import to_sqla_type
 from ibis.backends.base.sql.alchemy.registry import get_col_or_deferred_col
@@ -36,36 +31,6 @@ operation_registry = sqlalchemy_operation_registry.copy()
 operation_registry.update(sqlalchemy_window_functions_registry)
 
 
-# TODO: substr and find are copied from SQLite, we should really have a
-# "base" set of SQL functions that are the most common APIs across the major
-# RDBMS
-def _substr(t, expr):
-    f = sa.func.substr
-
-    arg, start, length = expr.op().args
-
-    sa_arg = t.translate(arg)
-    sa_start = t.translate(start)
-
-    if length is None:
-        return f(sa_arg, sa_start + 1)
-    else:
-        sa_length = t.translate(length)
-        return f(sa_arg, sa_start + 1, sa_length)
-
-
-def _string_find(t, expr):
-    arg, substr, start, _ = expr.op().args
-
-    if start is not None:
-        raise NotImplementedError
-
-    sa_arg = t.translate(arg)
-    sa_substr = t.translate(substr)
-
-    return sa.func.strpos(sa_arg, sa_substr) - 1
-
-
 def _extract(fmt, output_type=sa.SMALLINT):
     def translator(t, expr, output_type=output_type):
         (arg,) = expr.op().args
@@ -86,9 +51,9 @@ def _millisecond(t, expr):
     # we get total number of milliseconds including seconds with extract so we
     # mod 1000
     (sa_arg,) = map(t.translate, expr.op().args)
-    return (
-        sa.cast(sa.func.floor(sa.extract('millisecond', sa_arg)), sa.SMALLINT)
-        % 1000
+    return sa.cast(
+        sa.func.floor(sa.extract('millisecond', sa_arg)) % 1000,
+        sa.SMALLINT,
     )
 
 
@@ -138,33 +103,12 @@ def _is_inf(t, expr):
     return sa.or_(sa_arg == inf, sa_arg == -inf)
 
 
-def _cast(t, expr):
-    arg, typ = expr.op().args
-
-    sa_arg = t.translate(arg)
-    sa_type = t.get_sqla_type(typ)
-
-    # specialize going from an integer type to a timestamp
-    if isinstance(arg.type(), dt.Integer) and isinstance(sa_type, sa.DateTime):
-        return sa.func.timezone('UTC', sa.func.to_timestamp(sa_arg))
-
-    if arg.type().equals(dt.binary) and typ.equals(dt.string):
-        return sa.func.encode(sa_arg, 'escape')
-
-    if typ.equals(dt.binary):
-        #  decode yields a column of memoryview which is annoying to deal with
-        # in pandas. CAST(expr AS BYTEA) is correct and returns byte strings.
-        return sa.cast(sa_arg, sa.LargeBinary())
-
-    return sa.cast(sa_arg, sa_type)
-
-
 def _typeof(t, expr):
     (arg,) = expr.op().args
     sa_arg = t.translate(arg)
     typ = sa.cast(sa.func.pg_typeof(sa_arg), sa.TEXT)
 
-    # select pg_typeof('asdf') returns unknown so we have to check the child's
+    # select pg_typeof('thing') returns unknown so we have to check the child's
     # type for nullness
     return sa.case(
         [
@@ -175,20 +119,6 @@ def _typeof(t, expr):
     )
 
 
-def _string_agg(t, expr):
-    # we could use sa.func.string_agg since postgres 9.0, but we can cheaply
-    # maintain backwards compatibility here, so we don't use it
-    arg, sep, where = expr.op().args
-    sa_arg = t.translate(arg)
-    sa_sep = t.translate(sep)
-
-    if where is not None:
-        operand = t.translate(where.ifelse(arg, ibis.NA))
-    else:
-        operand = sa_arg
-    return sa.func.array_to_string(sa.func.array_agg(operand), sa_sep)
-
-
 _strftime_to_postgresql_rules = {
     '%a': 'TMDy',  # TM does it in a locale dependent way
     '%A': 'TMDay',
@@ -237,7 +167,7 @@ except AttributeError:
 # translate strftime spec into mostly equivalent PostgreSQL spec
 _scanner = re.Scanner(  # type: ignore # re does have a Scanner attribute
     # double quotes need to be escaped
-    [('"', lambda scanner, token: r'\"')]
+    [('"', lambda *_: r'\"')]
     + [
         (
             '|'.join(
@@ -258,7 +188,7 @@ _scanner = re.Scanner(  # type: ignore # re does have a Scanner attribute
                     ),
                 )
             ),
-            lambda scanner, token: token,
+            lambda _, token: token,
         )
     ]
 )
@@ -339,41 +269,23 @@ def _strftime(t, expr):
     arg, pattern = map(t.translate, expr.op().args)
     tokens, _ = _scanner.scan(pattern.value)
     reduced = _reduce_tokens(tokens, arg)
-    result = functools.reduce(sa.sql.ColumnElement.concat, reduced)
-    return result
-
-
-class _array_search(expression.FunctionElement):
-    type = sa.INTEGER()
-    name = 'array_search'
-
-
-@compiles(_array_search)
-def _postgresql_array_search(element, compiler, **kw):
-    needle, haystack = element.clauses
-    i = sa.func.generate_subscripts(haystack, 1).alias('i')
-    c0 = sa.column('i', type_=sa.INTEGER(), _selectable=i)
-    res = (
-        sa.select([c0])
-        .where(haystack[c0].op('IS NOT DISTINCT FROM')(needle))
-        .order_by(c0)
-        .limit(1)
-    )
-    subq = res.scalar_subquery()
-    result = sa.func.coalesce(subq, 0) - 1
-    string_result = compiler.process(result, **kw)
-    return string_result
+    return functools.reduce(sa.sql.ColumnElement.concat, reduced)
 
 
 def _find_in_set(t, expr):
-    # postgresql 9.5 has array_position, but the code below works on any
-    # version of postgres with generate_subscripts
-    # TODO: could make it even more generic by using generate_series
-    # TODO: this works with *any* type, not just strings. should the operation
-    #       itself also have this property?
-    needle, haystack = expr.op().args
-    return _array_search(
-        t.translate(needle), postgresql.array(list(map(t.translate, haystack)))
+    # TODO
+    # this operation works with any type, not just strings. should the
+    # operation itself also have this property?
+    op = expr.op()
+    return (
+        sa.func.coalesce(
+            sa.func.array_position(
+                postgresql.array(list(map(t.translate, op.values))),
+                t.translate(op.needle),
+            ),
+            0,
+        )
+        - 1
     )
 
 
@@ -384,22 +296,6 @@ def _regex_replace(t, expr):
     return sa.func.regexp_replace(string, pattern, replacement, 'g')
 
 
-def _reduction(func_name):
-    def reduction_compiler(t, expr):
-        arg, where = expr.op().args
-
-        if arg.type().equals(dt.boolean):
-            arg = arg.cast('int32')
-
-        func = getattr(sa.func, func_name)
-
-        if where is not None:
-            arg = where.ifelse(arg, None)
-        return func(t.translate(arg))
-
-    return reduction_compiler
-
-
 def _log(t, expr):
     arg, base = expr.op().args
     sa_arg = t.translate(arg)
@@ -414,36 +310,23 @@ def _log(t, expr):
     return sa.func.ln(sa_arg)
 
 
-class _regex_extract(GenericFunction):
-    def __init__(self, string, pattern, index):
-        super().__init__(string, pattern, index)
-        self.string = string
-        self.pattern = pattern
-        self.index = index
-
-
-@compiles(_regex_extract, 'postgresql')
-def _compile_regex_extract(element, compiler, **kw):
-    result = '(SELECT * FROM REGEXP_MATCHES({}, {}))[{}]'.format(
-        compiler.process(element.string, **kw),
-        compiler.process(element.pattern, **kw),
-        compiler.process(element.index, **kw),
-    )
-    return result
-
-
-def _regex_extract_(t, expr):
-    string, pattern, index = map(t.translate, expr.op().args)
-    result = sa.case(
+def _regex_extract(t, expr):
+    op = expr.op()
+    arg = t.translate(op.arg)
+    pattern = t.translate(op.pattern)
+    return sa.case(
         [
             (
-                sa.func.textregexeq(string, pattern),
-                sa.func._regex_extract(string, pattern, index + 1),
+                sa.func.textregexeq(arg, pattern),
+                sa.func.regexp_match(
+                    arg,
+                    pattern,
+                    type_=postgresql.ARRAY(sa.TEXT),
+                )[t.translate(op.index) + 1],
             )
         ],
-        else_='',
+        else_="",
     )
-    return result
 
 
 def _cardinality(array):
@@ -454,25 +337,18 @@ def _cardinality(array):
 
 
 def _array_repeat(t, expr):
-    """Is this really that useful?
-
-    Repeat an array like a Python list using modular arithmetic,
-    scalar subqueries, and PostgreSQL's ARRAY function.
-
-    This is inefficient if PostgreSQL allocates memory for the entire sequence
-    and the output column. A quick glance at PostgreSQL's C code shows the
-    sequence is evaluated stepwise, which suggests that it's roughly constant
-    memory for the sequence generation.
-    """
-    raw, times = map(t.translate, expr.op().args)
+    """Repeat an array."""
+    op = expr.op()
+    arg = t.translate(op.arg)
+    times = t.translate(op.times)
 
     # SQLAlchemy uses our column's table in the FROM clause. We need a simpler
     # expression to workaround this.
-    array = sa.column(raw.name, type_=raw.type)
+    array = sa.column(arg.name, type_=arg.type)
 
     # We still need to prefix the table name to the column name in the final
     # query, so make sure the column knows its origin
-    array.table = raw.table
+    array.table = arg.table
 
     array_length = _cardinality(array)
 
@@ -484,32 +360,15 @@ def _array_repeat(t, expr):
     series = sa.func.generate_series(start, stop, step).alias()
     series_column = sa.column(series.name, type_=sa.INTEGER)
 
-    # if our current index modulo the array's length
-    # is a multiple of the array's length, then the index is the array's length
+    # if our current index modulo the array's length is a multiple of the
+    # array's length, then the index is the array's length
     index_expression = series_column % array_length
     index = sa.func.coalesce(sa.func.nullif(index_expression, 0), array_length)
 
     # tie it all together in a scalar subquery and collapse that into an ARRAY
-    selected = sa.select([array[index]]).select_from(series)
-    subq = selected.scalar_subquery()
-    return sa.func.array(subq)
-
-
-def _identical_to(t, expr):
-    left, right = args = expr.op().args
-    if left.equals(right):
-        return True
-    else:
-        left, right = map(t.translate, args)
-        return left.op('IS NOT DISTINCT FROM')(right)
-
-
-def _hll_cardinality(t, expr):
-    # postgres doesn't have a builtin HLL algorithm, so we default to standard
-    # count distinct for now
-    arg, _ = expr.op().args
-    sa_arg = t.translate(arg)
-    return sa.func.count(sa.distinct(sa_arg))
+    return sa.func.array(
+        sa.select(array[index]).select_from(series).scalar_subquery()
+    )
 
 
 def _table_column(t, expr):
@@ -531,7 +390,7 @@ def _table_column(t, expr):
     # If the column does not originate from the table set in the current SELECT
     # context, we should format as a subquery
     if t.permit_subquery and ctx.is_foreign_expr(table):
-        return sa.select([out_expr])
+        return sa.select(out_expr)
 
     return out_expr
 
@@ -594,7 +453,7 @@ def _array_slice(t, expr):
     return sa_arg[sa_start + 1 : sa_stop]
 
 
-def _literal(t, expr):
+def _literal(_, expr):
     dtype = expr.type()
     op = expr.op()
     value = op.value
@@ -615,10 +474,6 @@ def _literal(t, expr):
         return sa.literal(value)
 
 
-def _random(t, expr):
-    return sa.func.random()
-
-
 def _day_of_week_index(t, expr):
     (sa_arg,) = map(t.translate, expr.op().args)
     return sa.cast(
@@ -631,13 +486,61 @@ def _day_of_week_name(t, expr):
     return sa.func.trim(sa.func.to_char(sa_arg, 'Day'))
 
 
+def _array_column(t, expr):
+    return postgresql.array(list(map(t.translate, expr.op().cols)))
+
+
+def _string_agg(t, expr):
+    op = expr.op()
+    agg = sa.func.string_agg(t.translate(op.arg), t.translate(op.sep))
+    if (where := op.where) is not None:
+        return agg.filter(t.translate(where))
+    return agg
+
+
+def _corr(t, expr):
+    if expr.op().how == "sample":
+        raise ValueError(
+            "PostgreSQL only implements population correlation coefficient"
+        )
+    return _binary_variance_reduction(sa.func.corr)(t, expr)
+
+
+def _covar(t, expr):
+    suffix = {"sample": "samp", "pop": "pop"}
+    how = suffix.get(expr.op().how, "samp")
+    func = getattr(sa.func, f"covar_{how}")
+    return _binary_variance_reduction(func)(t, expr)
+
+
+def _binary_variance_reduction(func):
+    def variance_compiler(t, expr):
+        op = expr.op()
+
+        x = op.left
+        if isinstance(x_type := x.type(), dt.Boolean):
+            x = x.cast(dt.Int32(nullable=x_type.nullable))
+
+        y = op.right
+        if isinstance(y_type := y.type(), dt.Boolean):
+            y = y.cast(dt.Int32(nullable=y_type.nullable))
+
+        result = func(t.translate(x), t.translate(y))
+
+        if (where := op.where) is not None:
+            result = result.filter(t.translate(where))
+
+        return result
+
+    return variance_compiler
+
+
 operation_registry.update(
     {
         ops.Literal: _literal,
         # We override this here to support time zones
         ops.TableColumn: _table_column,
         # types
-        ops.Cast: _cast,
         ops.TypeOf: _typeof,
         # Floating
         ops.IsNan: _is_nan,
@@ -650,14 +553,12 @@ operation_registry.update(
         ops.NotAny: unary(lambda x: sa.not_(sa.func.bool_or(x))),
         ops.NotAll: unary(lambda x: sa.not_(sa.func.bool_and(x))),
         # strings
-        ops.Substring: _substr,
-        ops.StringFind: _string_find,
         ops.GroupConcat: _string_agg,
         ops.Capitalize: unary(sa.func.initcap),
-        ops.RegexSearch: infix_op('~'),
+        ops.RegexSearch: fixed_arity(lambda x, y: x.op("~")(y), 2),
         ops.RegexReplace: _regex_replace,
         ops.Translate: fixed_arity('translate', 3),
-        ops.RegexExtract: _regex_extract_,
+        ops.RegexExtract: _regex_extract,
         ops.StringSplit: fixed_arity(
             lambda col, sep: sa.func.string_to_array(
                 col, sep, type_=sa.ARRAY(col.type)
@@ -676,12 +577,12 @@ operation_registry.update(
         ops.DateTruncate: _timestamp_truncate,
         ops.TimestampTruncate: _timestamp_truncate,
         ops.IntervalFromInteger: _interval_from_integer,
-        ops.DateAdd: infix_op('+'),
-        ops.DateSub: infix_op('-'),
-        ops.DateDiff: infix_op('-'),
-        ops.TimestampAdd: infix_op('+'),
-        ops.TimestampSub: infix_op('-'),
-        ops.TimestampDiff: infix_op('-'),
+        ops.DateAdd: fixed_arity(operator.add, 2),
+        ops.DateSub: fixed_arity(operator.sub, 2),
+        ops.DateDiff: fixed_arity(operator.sub, 2),
+        ops.TimestampAdd: fixed_arity(operator.add, 2),
+        ops.TimestampSub: fixed_arity(operator.sub, 2),
+        ops.TimestampDiff: fixed_arity(operator.sub, 2),
         ops.Strftime: _strftime,
         ops.ExtractYear: _extract('year'),
         ops.ExtractMonth: _extract('month'),
@@ -696,13 +597,6 @@ operation_registry.update(
         ops.ExtractMillisecond: _millisecond,
         ops.DayOfWeekIndex: _day_of_week_index,
         ops.DayOfWeekName: _day_of_week_name,
-        ops.Sum: _reduction('sum'),
-        ops.Mean: _reduction('avg'),
-        ops.Min: _reduction('min'),
-        ops.Max: _reduction('max'),
-        ops.Variance: variance_reduction('var'),
-        ops.StandardDev: variance_reduction('stddev'),
-        ops.RandomScalar: _random,
         # now is in the timezone of the server, but we want UTC
         ops.TimestampNow: lambda *_: sa.func.timezone('UTC', sa.func.now()),
         ops.TimeFromHMS: fixed_arity(sa.func.make_time, 3),
@@ -711,6 +605,7 @@ operation_registry.update(
         # array operations
         ops.ArrayLength: unary(_cardinality),
         ops.ArrayCollect: unary(sa.func.array_agg),
+        ops.ArrayColumn: _array_column,
         ops.ArraySlice: _array_slice,
         ops.ArrayIndex: fixed_arity(
             lambda array, index: array[_neg_idx_to_pos(array, index) + 1], 2
@@ -719,7 +614,8 @@ operation_registry.update(
             sa.sql.expression.ColumnElement.concat, 2
         ),
         ops.ArrayRepeat: _array_repeat,
-        ops.IdenticalTo: _identical_to,
-        ops.HLLCardinality: _hll_cardinality,
+        ops.Unnest: unary(sa.func.unnest),
+        ops.Covariance: _covar,
+        ops.Correlation: _corr,
     }
 )
