@@ -1,4 +1,4 @@
-from ibis.expr.datatypes.core import *  # noqa: F403
+from ibis.expr.datatypes.core import *  # noqa: F401,F403
 from ibis.expr.datatypes.core import _normalize, _WellKnownText  # noqa: F401
 
 halffloat = float16  # noqa: F405
