@@ -3,12 +3,12 @@ from types import FunctionType, LambdaType
 from public import public
 
 from ibis.expr import rules as rlz
-from ibis.expr.operations.analytic import AnalyticOp
-from ibis.expr.operations.core import ValueOp, distinct_roots
+from ibis.expr.operations.analytic import Analytic
+from ibis.expr.operations.core import Value, distinct_roots
 from ibis.expr.operations.reductions import Reduction
 
 
-class VectorizedUDF(ValueOp):
+class VectorizedUDF(Value):
     func = rlz.instance_of((FunctionType, LambdaType))
     func_args = rlz.tuple_of(rlz.column(rlz.any))
     # TODO(kszucs): should rename these arguments to
@@ -44,7 +44,7 @@ class ReductionVectorizedUDF(VectorizedUDF, Reduction):
 
 # TODO(kszucs): revisit
 @public
-class AnalyticVectorizedUDF(VectorizedUDF, AnalyticOp):
+class AnalyticVectorizedUDF(VectorizedUDF, Analytic):
     """Node for analytics UDF."""
 
     output_shape = rlz.Shape.COLUMNAR
