@@ -4,23 +4,24 @@ import collections
 import functools
 import itertools
 import operator
+from functools import cached_property
 from typing import IO, TYPE_CHECKING, Any, Iterable, Literal, Mapping, Sequence
 
 import numpy as np
 import tabulate
-from cached_property import cached_property
 from public import public
 
 import ibis
 from ibis import util
 from ibis.common import exceptions as com
+from ibis.expr.deferred import Deferred
 from ibis.expr.types.core import Expr
 
 if TYPE_CHECKING:
     from ibis.expr import schema as sch
     from ibis.expr import types as ir
-    from ibis.expr.types.generic import ColumnExpr
-    from ibis.expr.types.groupby import GroupedTableExpr
+    from ibis.expr.types.generic import Column
+    from ibis.expr.types.groupby import GroupedTable
 
 
 _ALIASES = (f"_ibis_view_{n:d}" for n in itertools.count())
@@ -40,16 +41,14 @@ def _regular_join_method(
     ],
 ):
     def f(
-        self: TableExpr,
-        right: TableExpr,
+        self: Table,
+        right: Table,
         predicates: str
         | Sequence[
-            str
-            | tuple[str | ir.ColumnExpr, str | ir.ColumnExpr]
-            | ir.BooleanValue
+            str | tuple[str | ir.Column, str | ir.Column] | ir.BooleanValue
         ] = (),
         suffixes: tuple[str, str] = ("_x", "_y"),
-    ) -> TableExpr:
+    ) -> Table:
         f"""Perform a{'n' * how.startswith(tuple("aeiou"))} {how} join between two tables.
 
         Parameters
@@ -64,7 +63,7 @@ def _regular_join_method(
 
         Returns
         -------
-        TableExpr
+        Table
             Joined table
         """  # noqa: E501
         return self.join(right, predicates, how=how, suffixes=suffixes)
@@ -74,20 +73,7 @@ def _regular_join_method(
 
 
 @public
-class TableExpr(Expr):
-    def _is_valid(self, exprs):
-        try:
-            self._assert_valid(util.promote_list(exprs))
-        except com.RelationError:
-            return False
-        else:
-            return True
-
-    def _assert_valid(self, exprs):
-        from ibis.expr.analysis import ExprValidator
-
-        ExprValidator([self]).validate_all(exprs)
-
+class Table(Expr):
     def __contains__(self, name):
         return name in self.schema()
 
@@ -98,8 +84,8 @@ class TableExpr(Expr):
         return self.execute()._repr_html_()
 
     def __getitem__(self, what):
-        from ibis.expr.types.analytic import AnalyticExpr
-        from ibis.expr.types.generic import ColumnExpr
+        from ibis.expr.types.analytic import TopK
+        from ibis.expr.types.generic import Column
         from ibis.expr.types.logical import BooleanColumn
 
         if isinstance(what, (str, int)):
@@ -122,16 +108,15 @@ class TableExpr(Expr):
 
         what = bind_expr(self, what)
 
-        if isinstance(what, AnalyticExpr):
-            what = what._table_getitem()
-
-        if isinstance(what, (list, tuple, TableExpr)):
+        if isinstance(what, TopK):
+            return what._to_semi_join(self)[self]
+        elif isinstance(what, (list, tuple, Table)):
             # Projection case
             return self.projection(what)
         elif isinstance(what, BooleanColumn):
             # Boolean predicate
             return self.filter([what])
-        elif isinstance(what, ColumnExpr):
+        elif isinstance(what, Column):
             # Projection convenience
             return self.projection(what)
         else:
@@ -163,27 +148,20 @@ class TableExpr(Expr):
     def __dir__(self):
         return sorted(frozenset(dir(type(self)) + self.columns))
 
-    # TODO(kszucs): should be removed
-    def _resolve(self, exprs):
-        exprs = util.promote_list(exprs)
-        return list(map(self._ensure_expr, exprs))
-
     # TODO(kszucs): should be removed
     def _ensure_expr(self, expr):
         if isinstance(expr, str):
             return self[expr]
         elif isinstance(expr, (int, np.integer)):
             return self[self.schema().name_at_position(expr)]
+        elif isinstance(expr, Deferred):
+            return expr.resolve(self)
         elif not isinstance(expr, Expr):
             return expr(self)
         else:
             return expr
 
-    # TODO(kszucs): should be removed
-    def _get_type(self, name):
-        return self._arg.get_type(name)
-
-    def get_columns(self, iterable: Iterable[str]) -> list[ColumnExpr]:
+    def get_columns(self, iterable: Iterable[str]) -> list[Column]:
         """
         Get multiple columns from the table
 
@@ -203,18 +181,18 @@ class TableExpr(Expr):
 
         Returns
         -------
-        list[ir.ColumnExpr]
+        list[ir.Column]
             List of column expressions
         """
         return [self.get_column(x) for x in iterable]
 
-    def get_column(self, name: str) -> ColumnExpr:
+    def get_column(self, name: str) -> Column:
         """
         Get a reference to a single column from the table
 
         Returns
         -------
-        ColumnExpr
+        Column
             A column named `name`.
         """
         import ibis.expr.operations as ops
@@ -241,7 +219,7 @@ class TableExpr(Expr):
         self,
         by=None,
         **additional_grouping_expressions: Any,
-    ) -> GroupedTableExpr:
+    ) -> GroupedTable:
         """
         Create a grouped table expression.
 
@@ -262,12 +240,12 @@ class TableExpr(Expr):
 
         Returns
         -------
-        GroupedTableExpr
+        GroupedTable
             A grouped table expression
         """
-        from ibis.expr.types.groupby import GroupedTableExpr
+        from ibis.expr.types.groupby import GroupedTable
 
-        return GroupedTableExpr(self, by, **additional_grouping_expressions)
+        return GroupedTable(self, by, **additional_grouping_expressions)
 
     groupby = group_by
 
@@ -300,21 +278,21 @@ class TableExpr(Expr):
 
         return ops.RowID().to_expr()
 
-    def view(self) -> TableExpr:
+    def view(self) -> Table:
         """Create a new table expression distinct from the current one.
 
         Use this API for any self-referencing operations like a self-join.
 
         Returns
         -------
-        TableExpr
+        Table
             Table expression
         """
         from ibis.expr import operations as ops
 
         return ops.SelfReference(self).to_expr()
 
-    def difference(self, right: TableExpr) -> TableExpr:
+    def difference(self, right: Table) -> Table:
         """Compute the set difference of two table expressions.
 
         The input tables must have identical schemas.
@@ -326,7 +304,7 @@ class TableExpr(Expr):
 
         Returns
         -------
-        TableExpr
+        Table
             The rows present in `left` that are not present in `right`.
         """
         from ibis.expr import operations as ops
@@ -335,11 +313,11 @@ class TableExpr(Expr):
 
     def aggregate(
         self,
-        metrics: Sequence[ir.ScalarExpr] | None = None,
-        by: Sequence[ir.ValueExpr] | None = None,
+        metrics: Sequence[ir.Scalar] | None = None,
+        by: Sequence[ir.Value] | None = None,
         having: Sequence[ir.BooleanValue] | None = None,
-        **kwargs: ir.ValueExpr,
-    ) -> TableExpr:
+        **kwargs: ir.Value,
+    ) -> Table:
         """Aggregate a table with a given set of reductions grouping by `by`.
 
         Parameters
@@ -355,7 +333,7 @@ class TableExpr(Expr):
 
         Returns
         -------
-        TableExpr
+        Table
             An aggregate table expression
         """
         metrics = [] if metrics is None else util.promote_list(metrics)
@@ -365,19 +343,34 @@ class TableExpr(Expr):
 
         op = self.op().aggregate(
             self,
-            metrics,
-            by=util.promote_list(by if by is not None else []),
-            having=util.promote_list(having if having is not None else []),
+            [
+                metric
+                if util.is_iterable(metric)
+                else self._ensure_expr(metric)
+                for metric in metrics
+            ],
+            by=list(
+                map(
+                    self._ensure_expr,
+                    util.promote_list(by if by is not None else []),
+                )
+            ),
+            having=list(
+                map(
+                    self._ensure_expr,
+                    util.promote_list(having if having is not None else []),
+                )
+            ),
         )
         return op.to_expr()
 
-    def distinct(self) -> TableExpr:
+    def distinct(self) -> Table:
         """Compute the set of unique rows in the table."""
         from ibis.expr import operations as ops
 
         return ops.Distinct(self).to_expr()
 
-    def limit(self, n: int, offset: int = 0) -> TableExpr:
+    def limit(self, n: int, offset: int = 0) -> Table:
         """Select the first `n` rows at beginning of table starting at `offset`.
 
         Parameters
@@ -389,7 +382,7 @@ class TableExpr(Expr):
 
         Returns
         -------
-        TableExpr
+        Table
             The first `n` rows of `table` starting at `offset`
         """
         from ibis.expr import operations as ops
@@ -397,7 +390,7 @@ class TableExpr(Expr):
         op = ops.Limit(self, n, offset=offset)
         return op.to_expr()
 
-    def head(self, n: int = 5) -> TableExpr:
+    def head(self, n: int = 5) -> Table:
         """Select the first `n` rows of a table.
 
         The result set is not deterministic without a sort.
@@ -409,7 +402,7 @@ class TableExpr(Expr):
 
         Returns
         -------
-        TableExpr
+        Table
             `table` limited to `n` rows
         """
         return self.limit(n=n)
@@ -417,11 +410,11 @@ class TableExpr(Expr):
     def sort_by(
         self,
         sort_exprs: str
-        | ir.ColumnExpr
+        | ir.Column
         | ir.SortKey
-        | tuple[str | ir.ColumnExpr, bool]
-        | Sequence[tuple[str | ir.ColumnExpr, bool]],
-    ) -> TableExpr:
+        | tuple[str | ir.Column, bool]
+        | Sequence[tuple[str | ir.Column, bool]],
+    ) -> Table:
         """Sort table by `sort_exprs`
 
         Parameters
@@ -437,7 +430,7 @@ class TableExpr(Expr):
 
         Returns
         -------
-        TableExpr
+        Table
             Sorted table
         """
         if isinstance(sort_exprs, tuple):
@@ -450,9 +443,9 @@ class TableExpr(Expr):
 
     def union(
         self,
-        right: TableExpr,
+        right: Table,
         distinct: bool = False,
-    ) -> TableExpr:
+    ) -> Table:
         """Compute the set union of two table expressions.
 
         The input tables must have identical schemas.
@@ -467,14 +460,14 @@ class TableExpr(Expr):
 
         Returns
         -------
-        TableExpr
+        Table
             Union of table and `right`
         """
         from ibis.expr import operations as ops
 
         return ops.Union(self, right, distinct=distinct).to_expr()
 
-    def intersect(self, right: TableExpr) -> TableExpr:
+    def intersect(self, right: Table) -> Table:
         """Compute the set intersection of two table expressions.
 
         The input tables must have identical schemas.
@@ -486,19 +479,19 @@ class TableExpr(Expr):
 
         Returns
         -------
-        TableExpr
+        Table
             The rows common amongst `left` and `right`.
         """
         from ibis.expr import operations as ops
 
         return ops.Intersection(self, right).to_expr()
 
-    def to_array(self) -> ir.ColumnExpr:
+    def to_array(self) -> ir.Column:
         """View a single column table as an array.
 
         Returns
         -------
-        ValueExpr
+        Value
             A single column view of a table
         """
         from ibis.expr import operations as ops
@@ -511,17 +504,11 @@ class TableExpr(Expr):
 
         return ops.TableArrayView(self).to_expr()
 
-    def _safe_get_name(expr):
-        try:
-            return expr.get_name()
-        except com.ExpressionError:
-            return None
-
     def mutate(
         self,
         exprs: Sequence[ir.Expr] | None = None,
-        **mutations: ir.ValueExpr,
-    ) -> TableExpr:
+        **mutations: ir.Value,
+    ) -> Table:
         """Add columns to a table expression.
 
         Parameters
@@ -533,7 +520,7 @@ class TableExpr(Expr):
 
         Returns
         -------
-        TableExpr
+        Table
             Table expression with additional columns
 
         Examples
@@ -556,7 +543,7 @@ class TableExpr(Expr):
             baz: 5
             qux: r0.foo + r0.bar
 
-        Use the [`name`][ibis.expr.types.generic.ValueExpr.name] method to name
+        Use the [`name`][ibis.expr.types.generic.Value.name] method to name
         the new columns.
 
         >>> new_columns = [ibis.literal(5).name('baz',),
@@ -575,6 +562,8 @@ class TableExpr(Expr):
         ):
             if util.is_function(expr):
                 value = expr(self)
+            elif isinstance(expr, Deferred):
+                value = expr.resolve(self)
             else:
                 value = rlz.any(expr)
             exprs.append(value.name(name))
@@ -584,8 +573,8 @@ class TableExpr(Expr):
 
     def select(
         self,
-        exprs: ir.ValueExpr | str | Sequence[ir.ValueExpr | str],
-    ) -> TableExpr:
+        exprs: ir.Value | str | Sequence[ir.Value | str],
+    ) -> Table:
         """Compute a new table expression using `exprs`.
 
         Passing an aggregate function to this method will broadcast the
@@ -601,7 +590,7 @@ class TableExpr(Expr):
 
         Returns
         -------
-        TableExpr
+        Table
             Table expression
 
         Examples
@@ -633,8 +622,8 @@ class TableExpr(Expr):
           b float64
         Selection[r0]
           selections:
-            sum_a:  WindowOp(Sum(r0.a), window=Window(how='rows'))
-            mean_b: WindowOp(Mean(r0.b), window=Window(how='rows'))
+            sum_a:  Window(Sum(r0.a), window=Window(how='rows'))
+            mean_b: Window(Mean(r0.b), window=Window(how='rows'))
 
         Note the `Window` objects here.
 
@@ -649,7 +638,7 @@ class TableExpr(Expr):
           b float64
         Selection[r0]
           selections:
-            demeaned_a: r0.a - WindowOp(Mean(r0.a), window=Window(how='rows'))
+            demeaned_a: r0.a - Window(Mean(r0.a), window=Window(how='rows'))
         """
         import ibis.expr.analysis as an
 
@@ -662,7 +651,7 @@ class TableExpr(Expr):
 
     projection = select
 
-    def relabel(self, substitutions: Mapping[str, str]) -> TableExpr:
+    def relabel(self, substitutions: Mapping[str, str]) -> Table:
         """Change table column names, otherwise leaving table unaltered.
 
         Parameters
@@ -672,7 +661,7 @@ class TableExpr(Expr):
 
         Returns
         -------
-        TableExpr
+        Table
             A relabeled table expression
         """
         observed = set()
@@ -691,7 +680,7 @@ class TableExpr(Expr):
 
         return self.projection(exprs)
 
-    def drop(self, fields: str | Sequence[str]) -> TableExpr:
+    def drop(self, fields: str | Sequence[str]) -> Table:
         """Remove fields from a table.
 
         Parameters
@@ -701,7 +690,7 @@ class TableExpr(Expr):
 
         Returns
         -------
-        TableExpr
+        Table
             Expression without `fields`
         """
         if not fields:
@@ -722,7 +711,7 @@ class TableExpr(Expr):
     def filter(
         self,
         predicates: ir.BooleanValue | Sequence[ir.BooleanValue],
-    ) -> TableExpr:
+    ) -> Table:
         """Select rows from `table` based on `predicates`.
 
         Parameters
@@ -732,13 +721,22 @@ class TableExpr(Expr):
 
         Returns
         -------
-        TableExpr
+        Table
             Filtered table expression
         """
         from ibis.expr import analysis as an
 
-        resolved_predicates = _resolve_predicates(self, predicates)
-        return an.apply_filter(self, resolved_predicates)
+        resolved_predicates, top_ks = _resolve_predicates(self, predicates)
+        child = self
+        for predicate, right in top_ks:
+            child = child.semi_join(right, predicate)[child]
+        return an.apply_filter(
+            child,
+            [
+                an._rewrite_filter(pred.op(), pred)
+                for pred in resolved_predicates
+            ],
+        )
 
     def count(self) -> ir.IntegerScalar:
         """Compute the number of rows in the table.
@@ -756,7 +754,7 @@ class TableExpr(Expr):
         self,
         subset: Sequence[str] | None = None,
         how: Literal["any", "all"] = "any",
-    ) -> TableExpr:
+    ) -> Table:
         """Remove rows with null values from the table.
 
         Parameters
@@ -779,7 +777,7 @@ class TableExpr(Expr):
 
         Returns
         -------
-        TableExpr
+        Table
             Table expression
         """
         from ibis.expr import operations as ops
@@ -791,8 +789,8 @@ class TableExpr(Expr):
 
     def fillna(
         self,
-        replacements: ir.ScalarExpr | Mapping[str, ir.ScalarExpr],
-    ) -> TableExpr:
+        replacements: ir.Scalar | Mapping[str, ir.Scalar],
+    ) -> Table:
         """Fill null values in a table expression.
 
         Parameters
@@ -827,7 +825,7 @@ class TableExpr(Expr):
 
         Returns
         -------
-        TableExpr
+        Table
             Table expression
         """  # noqa: E501
         from ibis.expr import operations as ops
@@ -874,7 +872,7 @@ class TableExpr(Expr):
         footer_line = "-" * width
         print("\n".join([tabulated, footer_line, row_count]), file=buf)
 
-    def set_column(self, name: str, expr: ir.ValueExpr) -> TableExpr:
+    def set_column(self, name: str, expr: ir.Value) -> Table:
         """Replace an existing column with a new expression.
 
         Parameters
@@ -886,7 +884,7 @@ class TableExpr(Expr):
 
         Returns
         -------
-        TableExpr
+        Table
             Table expression with new columns
         """
         expr = self._ensure_expr(expr)
@@ -907,13 +905,11 @@ class TableExpr(Expr):
         return self.projection(proj_exprs)
 
     def join(
-        left: TableExpr,
-        right: TableExpr,
+        left: Table,
+        right: Table,
         predicates: str
         | Sequence[
-            str
-            | tuple[str | ir.ColumnExpr, str | ir.ColumnExpr]
-            | ir.BooleanColumn
+            str | tuple[str | ir.Column, str | ir.Column] | ir.BooleanColumn
         ] = (),
         how: Literal[
             'inner',
@@ -928,7 +924,7 @@ class TableExpr(Expr):
         ] = 'inner',
         *,
         suffixes: tuple[str, str] = ("_x", "_y"),
-    ) -> TableExpr:
+    ) -> Table:
         """Perform a join between two tables.
 
         Parameters
@@ -981,16 +977,16 @@ class TableExpr(Expr):
         )
 
     def asof_join(
-        left: TableExpr,
-        right: TableExpr,
+        left: Table,
+        right: Table,
         predicates: str
         | ir.BooleanColumn
         | Sequence[str | ir.BooleanColumn] = (),
-        by: str | ir.ColumnExpr | Sequence[str | ir.ColumnExpr] = (),
+        by: str | ir.Column | Sequence[str | ir.Column] = (),
         tolerance: str | ir.IntervalScalar | None = None,
         *,
         suffixes: tuple[str, str] = ("_x", "_y"),
-    ) -> TableExpr:
+    ) -> Table:
         """Perform an "as-of" join between `left` and `right`.
 
         Similar to a left join except that the match is done on nearest key
@@ -1016,7 +1012,7 @@ class TableExpr(Expr):
 
         Returns
         -------
-        TableExpr
+        Table
             Table expression
         """
         from ibis.expr import operations as ops
@@ -1036,11 +1032,11 @@ class TableExpr(Expr):
         )
 
     def cross_join(
-        left: TableExpr,
-        right: TableExpr,
-        *rest: TableExpr,
+        left: Table,
+        right: Table,
+        *rest: Table,
         suffixes: tuple[str, str] = ("_x", "_y"),
-    ) -> TableExpr:
+    ) -> Table:
         """Compute the cross join of a sequence of tables.
 
         Parameters
@@ -1057,7 +1053,7 @@ class TableExpr(Expr):
 
         Returns
         -------
-        TableExpr
+        Table
             Cross join of `left`, `right` and `rest`
 
         Examples
@@ -1088,7 +1084,7 @@ class TableExpr(Expr):
 
         expr = ops.CrossJoin(
             left,
-            functools.reduce(TableExpr.cross_join, rest, right),
+            functools.reduce(Table.cross_join, rest, right),
             [],
         ).to_expr()
         return ops.relations._dedup_join_columns(
@@ -1098,7 +1094,8 @@ class TableExpr(Expr):
             suffixes=suffixes,
         )
 
-    def prevent_rewrite(self, client=None) -> TableExpr:
+    @util.deprecated(version="4.0", instead="")
+    def prevent_rewrite(self, client=None) -> Table:  # pragma: no cover
         """Prevent optimization from happening below this expression.
 
         Only valid on SQL-string generating backends.
@@ -1112,7 +1109,7 @@ class TableExpr(Expr):
 
         Returns
         -------
-        TableExpr
+        Table
             An opaque SQL query
         """
         from ibis.expr import operations as ops
@@ -1135,15 +1132,15 @@ class TableExpr(Expr):
         version="3.0",
         instead="remove the `.materialize()` call, it has no effect",
     )
-    def materialize(self) -> TableExpr:
+    def materialize(self) -> Table:
         return self
 
-    def alias(self, alias: str) -> ir.TableExpr:
+    def alias(self, alias: str) -> ir.Table:
         """Create a table expression with a specific name `alias`.
 
         This method is useful for exposing an ibis expression to the underlying
         backend for use in the
-        [`TableExpr.sql`][ibis.expr.types.relations.TableExpr.sql] method.
+        [`Table.sql`][ibis.expr.types.relations.Table.sql] method.
 
         !!! note "`.alias` will create a temporary view"
 
@@ -1159,7 +1156,7 @@ class TableExpr(Expr):
 
         Returns
         -------
-        TableExpr
+        Table
             An table expression
 
         Examples
@@ -1192,7 +1189,7 @@ class TableExpr(Expr):
         expr.compile()
         return expr
 
-    def sql(self, query: str) -> ir.TableExpr:
+    def sql(self, query: str) -> ir.Table:
         """Run a SQL query against a table expression.
 
         !!! note "The SQL string is backend specific"
@@ -1200,7 +1197,7 @@ class TableExpr(Expr):
             `query` must be valid SQL for the execution backend the expression
             will run against
 
-        See [`TableExpr.alias`][ibis.expr.types.relations.TableExpr.alias] for
+        See [`Table.alias`][ibis.expr.types.relations.Table.alias] for
         details on using named table expressions in a SQL string.
 
         Parameters
@@ -1210,7 +1207,7 @@ class TableExpr(Expr):
 
         Returns
         -------
-        TableExpr
+        Table
             An opaque table expression
 
         Examples
@@ -1237,21 +1234,32 @@ class TableExpr(Expr):
         ).to_expr()
 
 
-def _resolve_predicates(table: TableExpr, predicates) -> list[ir.BooleanValue]:
+def _resolve_predicates(
+    table: Table, predicates
+) -> tuple[list[ir.BooleanValue], list[tuple[ir.BooleanValue, ir.Table]]]:
     from ibis.expr import analysis as an
+    from ibis.expr import operations as ops
     from ibis.expr import types as ir
 
     if isinstance(predicates, Expr):
         predicates = an.flatten_predicate(predicates)
-    predicates = util.promote_list(predicates)
-    predicates = [ir.relations.bind_expr(table, x) for x in predicates]
-    resolved_predicates = []
-    for pred in predicates:
-        if isinstance(pred, ir.AnalyticExpr):
-            pred = pred.to_filter()
-        resolved_predicates.append(pred)
 
-    return resolved_predicates
+    predicates = [
+        ir.relations.bind_expr(table, pred)
+        for pred in util.promote_list(predicates)
+    ]
+
+    resolved_predicates = []
+    top_ks = []
+    for pred in predicates:
+        if isinstance(pred, ir.TopK):
+            top_ks.append(pred._semi_join_components())
+        elif isinstance(pred_op := pred.op(), ops.logical._UnresolvedSubquery):
+            resolved_predicates.append(pred_op._resolve(table))
+        else:
+            resolved_predicates.append(pred)
+
+    return resolved_predicates, top_ks
 
 
 def bind_expr(table, expr):
@@ -1261,13 +1269,14 @@ def bind_expr(table, expr):
     return table._ensure_expr(expr)
 
 
-# TODO: move to analysis
-def find_base_table(expr):
-    if isinstance(expr, TableExpr):
-        return expr
+@util.deprecated(
+    version="4.0",
+    instead="Use ibis.expr.analysis.find_first_base_table() instead",
+)
+def find_base_table(expr):  # pragma: no cover
+    from ibis.expr.analysis import find_first_base_table
 
-    for arg in expr.op().flat_args():
-        if isinstance(arg, Expr):
-            r = find_base_table(arg)
-            if isinstance(r, TableExpr):
-                return r
+    return find_first_base_table(expr)
+
+
+public(TableExpr=Table)
