@@ -16,7 +16,6 @@ from typing import Optional
 
 import pytest
 
-import ibis.expr.types as ir
 from ibis.backends.base.sql import BaseSQLBackend
 from ibis.backends.base.sql.alchemy import (
     AlchemyCompiler,
@@ -32,14 +31,27 @@ MOCK_TABLES = {
         ('b', 'int16'),
         ('c', 'int32'),
         ('d', 'int64'),
-        ('e', 'float'),
-        ('f', 'double'),
+        ('e', 'float32'),
+        ('f', 'float64'),
         ('g', 'string'),
         ('h', 'boolean'),
         ('i', 'timestamp'),
         ('j', 'date'),
         ('k', 'time'),
     ],
+    "foo_t": [('key1', 'string'), ('key2', 'string'), ('value1', 'double')],
+    "bar_t": [('key1', 'string'), ('key2', 'string'), ('value2', 'double')],
+    "t1": [('key1', 'string'), ('key2', 'string'), ('value1', 'double')],
+    "t2": [('key1', 'string'), ('key2', 'string'), ('value2', 'double')],
+    "t3": [('key2', 'string'), ('key3', 'string'), ('value3', 'double')],
+    "t4": [('key3', 'string'), ('value4', 'double')],
+    "bar": [('x', 'double'), ('job', 'string')],
+    "foo": [
+        ('job', 'string'),
+        ('dept_id', 'string'),
+        ('year', 'int32'),
+        ('y', 'double'),
+    ],
     'star1': [
         ('c', 'int32'),
         ('f', 'double'),
@@ -111,8 +123,8 @@ MOCK_TABLES = {
         ('smallint_col', 'int16'),
         ('int_col', 'int32'),
         ('bigint_col', 'int64'),
-        ('float_col', 'float'),
-        ('double_col', 'double'),
+        ('float_col', 'float32'),
+        ('double_col', 'float64'),
         ('date_string_col', 'string'),
         ('string_col', 'string'),
         ('timestamp_col', 'timestamp'),
@@ -394,6 +406,7 @@ class MockBackend(BaseSQLBackend):
 
 class MockAlchemyBackend(MockBackend):
     compiler = AlchemyCompiler
+    table_class = AlchemyTable
 
     def __init__(self):
         super().__init__()
@@ -405,13 +418,14 @@ class MockAlchemyBackend(MockBackend):
         return self._inject_table(name, schema)
 
     def _inject_table(self, name, schema):
-        if name in self.meta.tables:
-            table = self.meta.tables[name]
-        else:
-            table = table_from_schema(name, self.meta, schema)
+        try:
+            alchemy_table = self.meta.tables[name]
+        except KeyError:
+            alchemy_table = table_from_schema(name, self.meta, schema)
 
-        node = AlchemyTable(table, self)
-        return ir.TableExpr(node)
+        return self.table_class(
+            source=self, sqla_table=alchemy_table, schema=schema
+        ).to_expr()
 
 
 GEO_TABLE = {
