@@ -2,10 +2,12 @@ import enum
 
 import parsy
 import pytest
+from pytest import param
 from toolz import identity
 
 import ibis
 import ibis.expr.datatypes as dt
+import ibis.expr.operations as ops
 import ibis.expr.rules as rlz
 import ibis.expr.types as ir
 from ibis.common.exceptions import IbisTypeError
@@ -56,7 +58,7 @@ def test_valid_instance_of(klass, value, expected):
 @pytest.mark.parametrize(
     ('klass', 'value', 'expected'),
     [
-        (ir.TableExpr, object, IbisTypeError),
+        (ir.Table, object, IbisTypeError),
         (ir.IntegerValue, 4, IbisTypeError),
     ],
 )
@@ -191,33 +193,53 @@ def test_invalid_member_of(obj, value, expected):
 @pytest.mark.parametrize(
     ('validator', 'values', 'expected'),
     [
-        (rlz.value_list_of(identity), (3, 2), ibis.sequence([3, 2])),
-        (rlz.value_list_of(rlz.integer), (3, 2), ibis.sequence([3, 2])),
-        (
+        param(
+            rlz.value_list_of(identity),
+            (3, 2),
+            ibis.sequence([3, 2]),
+            id="list_identity",
+        ),
+        param(
+            rlz.value_list_of(rlz.integer),
+            (3, 2),
+            ibis.sequence([3, 2]),
+            id="list_int",
+        ),
+        param(
             rlz.value_list_of(rlz.integer),
             (3, None),
             ibis.sequence([3, ibis.NA]),
+            id="list_int_null",
         ),
-        (rlz.value_list_of(rlz.string), ('a',), ibis.sequence(['a'])),
-        (rlz.value_list_of(rlz.string), ['a', 'b'], ibis.sequence(['a', 'b'])),
-        pytest.param(
+        param(
+            rlz.value_list_of(rlz.string),
+            ('a',),
+            ibis.sequence(['a']),
+            id="list_string_one",
+        ),
+        param(
+            rlz.value_list_of(rlz.string),
+            ['a', 'b'],
+            ibis.sequence(['a', 'b']),
+            id="list_string_two",
+        ),
+        param(
             rlz.value_list_of(rlz.value_list_of(rlz.string)),
             [[], ['a']],
-            ibis.sequence([[], ['a']]),
-            marks=pytest.mark.xfail(
-                raises=ValueError, reason='Not yet implemented'
-            ),
+            ibis.sequence([ibis.sequence([]), ibis.sequence(['a'])]),
+            id="list_list_string",
         ),
-        (
+        param(
             rlz.value_list_of(rlz.boolean, min_length=2),
             [True, False],
             ibis.sequence([True, False]),
+            id="boolean",
         ),
     ],
 )
 def test_valid_value_list_of(validator, values, expected):
     result = validator(values)
-    assert isinstance(result, ir.ListExpr)
+    assert isinstance(result, ir.ValueList)
     assert len(result) == len(values)
     for a, b in zip(result, expected):
         assert a.equals(b)
@@ -397,3 +419,14 @@ def test_optional(validator, input):
     else:
         assert rlz.optional(validator)(input) == expected
     assert rlz.optional(validator)(None) is None
+
+
+def test_base_table_of_failure_mode():
+    class BrokenUseOfBaseTableOf(ops.Node):
+        arg = rlz.any
+        foo = rlz.function_of(rlz.base_table_of("arg"))
+
+    arg = ibis.literal("abc")
+
+    with pytest.raises(IbisTypeError, match="doesn't have a base table"):
+        BrokenUseOfBaseTableOf(arg, identity)
